<?php

namespace SuperRhino\AppStoreServerApi\Response;

use Carbon\Carbon;
use SuperRhino\AppStoreServerApi\Enum\EnvironmentEnum;

class JWSRenewalInfo extends AbstractResponse implements EnvironmentInterface {

    /**
     * The product identifier of the product that renews at the next billing period.
     */
    public string $autoRenewProductId;

    /**
     * The renewal status for an auto-renewable subscription.
     * 0: Automatic renewal is off. The customer has turned off automatic renewal for the subscription, and it won’t renew at the end of the current subscription period.
     * 1: Automatic renewal is on. The subscription renews at the end of the current subscription period.
     */
    public int $autoRenewStatus;

    /**
     * The server environment, either sandbox or production.
     */
    public string $environment;

    /**
     * The reason a subscription expired.
     * 1:The customer canceled their subscription.
     * 2:Billing error; for example, the customer’s payment information is no longer valid.
     * 3:The customer didn’t consent to an auto-renewable subscription price increase that requires customer consent, allowing the subscription to expire.
     * 4:The product wasn’t available for purchase at the time of renewal.
     */
    public int $expirationIntent;

    /**
     * The time when the billing grace period for subscription renewals expires.
     */
    public Carbon $gracePeriodExpiresDate;

    /**
     * The Boolean value that indicates whether the App Store is attempting to automatically renew an expired subscription.
     */
    public bool $isInBillingRetryPeriod;

    /**
     * The offer code or the promotional offer identifier.
     */
    public string $offerIdentifier;

    /**
     * A value that represents the promotional offer type.
     * 1: An introductory offer.
     * 2: A promotional offer.
     * 3: An offer with a subscription offer code.
     * The offer types 2 and 3 have an offerIdentifier.
     */
    public int $offerType;

    /**
     * The original transaction identifier of a purchase.
     */
    public string $originalTransactionId;

    /**
     * The status that indicates whether the auto-renewable subscription is subject to a price increase.
     * 0: The customer hasn’t yet responded to an auto-renewable subscription price increase that requires customer consent.
     * 1: The customer consented to an auto-renewable subscription price increase that requires customer consent, or the App Store has notified the customer of an auto-renewable subscription price increase that doesn’t require consent.
     */
    public int $priceIncreaseStatus;

    /**
     * The product identifier of the in-app purchase.
     */
    public string $productId;

    /**
     * The earliest start date of an auto-renewable subscription in a series of subscription purchases that ignores all lapses of paid service that are 60 days or less.
     */
    public Carbon $recentSubscriptionStartDate;

    /**
     * The App Store signed the JSON Web Signature data.
     */
    public Carbon $signedDate;

    public function parameterConverter($name, $value) {
        switch ($name) {
            case 'gracePeriodExpiresDate':
            case 'recentSubscriptionStartDate':
            case 'signedDate':
                return Carbon::createFromTimestampMs($value);
            default:
                return $value;
        }
    }

    public function isSandbox(): bool {
        return $this->environment == EnvironmentEnum::SANDBOX;
    }

    public function isProduction(): bool {
        return $this->environment == EnvironmentEnum::PRODUCTION;
    }

}